import { Body, Controller, Get, Post, Put, Query } from '@nestjs/common';
import { CarritoService } from './carrito.service';
import { CarritoCreateDto } from './dto/carrito-create.dto';
import { CarritoUpdateDto } from './dto/carrito-update.dto';
import { CarritoGetDto } from './dto/carrito-get-dto';
import { CarritoDto } from './dto/carrito.dto'
import {ApiOperation, ApiTags} from '@nestjs/swagger'

@Controller('carrito')
@ApiTags('carrito')
export class CarritoController {
  constructor(private carritoService: CarritoService) {}

  @Post()
  @ApiOperation({
    description: 'Creamos un carrito, pasandole los parametros id_tipo_carrito y id_modulo'
  })
  create(@Body() body: CarritoCreateDto) {
    return this.carritoService.create(
      body.id_tipo_carrito,
      body.id_modulo,
    );
  }

  @Get()
  @ApiOperation({
    description: 'Nos trae todos los carritos'
  })
  get() {
    return this.carritoService.findAll();
  }

  @Get('carrito')
  @ApiOperation({
    description: 'Nos trae un carrito en especifico, pasandole el parámetro id_carrito'
  })
  carrito(@Query() query: CarritoDto) {
    return this.carritoService.findById(parseInt(query.id_carrito));
  }

  @Get('carritos')
  @ApiOperation({
    description: 'Nos trae todos los carritos de un módulo, con el parámetro del id_modulo'
  })
  carritos(@Query() query: CarritoGetDto) {
    return this.carritoService.findByIdModulo(query);
  }

  @Get('carritoInstitucion')
  carritosInstitucion() {}

  @Put()
  @ApiOperation({
    description: 'Actualizamos información de un carrito'
  })
  update(@Body() body: CarritoUpdateDto) {
    return this.carritoService.update(body);
  }
}
