import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrera } from 'src/carrera/entity/carrera.entity';
import { Institucion } from 'src/institucion/entity/institucion.entity';
import { InstitucionCarrera } from './entity/institucion-carrera.entity';
import { CarreraService } from '../carrera/carrera.service';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionCarreraService {
  constructor(
    @InjectRepository(InstitucionCarrera)
    private repository: Repository<InstitucionCarrera>,
    private carreraService: CarreraService,
    private institucionService: InstitucionService,
  ) {}

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) => this.repository.find({ institucion }));
  }

  findById(id_institucion_carrera: number) {
    return this.repository
      .findOne({ id_institucion_carrera })
      .then((institucionCarrera) => {
        if (!institucionCarrera)
          throw new NotFoundException('No existe esta institución carrera.');
        return institucionCarrera;
      });
  }

  async findByIdInstitucionIdCarrera(
    id_institucion: number | Institucion,
    id_carrera: number | Carrera,
  ) {
    const institucion =
      typeof id_institucion === 'number'
        ? await this.institucionService.findById(id_institucion)
        : id_institucion;
    const carrera =
      typeof id_carrera === 'number'
        ? await this.institucionService.findById(id_carrera)
        : id_carrera;

    return this.repository.findOne({ carrera, institucion });
  }
}
