import { ConflictException, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { InstitucionTipoEntrada } from './entity/institucion-tipo-entrada.entity';
import { TipoEntrada } from './entity/tipo-entrada.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { Institucion } from '../institucion/entity/institucion.entity';

@Injectable()
export class InstitucionTipoEntradaService {
  constructor(
    @InjectRepository(InstitucionTipoEntrada)
    private repositoryInstitucionTipoEntrada: Repository<InstitucionTipoEntrada>,
    @InjectRepository(TipoEntrada)
    private repositoryTipoEntrada: Repository<TipoEntrada>,
    private institucionService: InstitucionService,
  ) {}

  async create(tipo_entrada: string) {
    // corregir
    return this.repositoryTipoEntrada
      .findOne({ tipo_entrada })
      .then((existeTipoEntrada) => {
        if (existeTipoEntrada)
          throw new ConflictException('Ya existe este tipo de entrada.');
        return this.repositoryTipoEntrada.save(
          this.repositoryTipoEntrada.create({
            tipo_entrada,
          }),
        );
      })
      .then(() => ({ message: 'Se creo correctamente la nueva entrada.' }));
  }

  findAll() {
    return this.repositoryTipoEntrada.find();
  }

  findAllByIdInstitucion(id_institucion: number, mostrar = false) {
    const busqueda: { institucion?: Institucion; mostrar?: boolean } = {};

    if (mostrar) busqueda.mostrar = mostrar;
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) => {
        busqueda.institucion = institucion;
        return this.repositoryInstitucionTipoEntrada.find(busqueda);
      });
  }

  findAllByIdInstitucionMostrar(id_institucion: number) {
    return this.findAllByIdInstitucion(id_institucion, true);
  }
}
